/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ref_count.cc,v 1.13 2009/01/01 07:46:43 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/ref_count.h"


#ifdef REF_TEST

#include <iostream>
#include <list>
#include <string>


class Test : public Ochusha::RefCount
{
private:
  string str;

public:
  Test(const char *text) { str = text; }
  ~Test() { cout << str << " deleted" << endl; }
  inline string text() { return str; }
};


int
main()
{
  Ochusha::smart_ptr<Test> foo = new Test("foo");
  Ochusha::smart_ptr<Test> bar = new Test("bar");
  Ochusha::smart_ptr<Test> baz = new Test("baz");

  typedef Ochusha::smart_ptr<Test> test_ptr;

  list<test_ptr> test_list;

  cout << foo->text() << endl;
  cout << bar->text() << endl;
  cout << baz->text() << endl;

  test_list.push_front(baz);

  cerr << "baz == foo: " << (baz == foo) << endl;

  baz = foo;

  cerr << "baz == foo: " << (baz == foo) << endl;

  cout << baz->text() << endl;

  bar = NULL;

  cout << test_list.back()->text() << endl;

  test_list.pop_back();

  cout << foo->text() << endl;

  return 0;
}
#endif
